// Определения, на каких пинах находятся лампы и дешифратор. L1-L6 лампы, A-D дешифратор.
#define L1            4
#define L2            3
#define L3            2
#define L4            1
#define L5            0
#define L6            16

#define A             15
#define B             14
#define C             9
#define D             10

#define DAT           20
#define SH            5
#define ST            21

#include <Wire.h>
#include <EncButton.h>
EncButton <EB_TICK, 13, 12, 11> enc2;  // энкодер с кнопкой <A, B, KEY>
EncButton <EB_TICK, 7,  6,  8> enc1;  // энкодер с кнопкой <A, B, KEY>

uint16_t freq = 1030; // 107.3FM
uint16_t EEMEM freq_addr;

uint8_t main_volume = 3;
uint8_t EEMEM main_volume_addr;

#include "defines_radio.h"

// переключатели текущей цифры и яркости
int a = 0;
int e = 0;

unsigned char dig [6] = {1, 0, 2, 5, 0, 1};

void setup() {
  main_volume = eeprom_read_byte(&main_volume_addr);
  freq = eeprom_read_word (&freq_addr);
  freq_to_lamps ();
  
  if ((main_volume > 13) || (main_volume < 0)) main_volume = 3;
  if ((freq > 1080) || (freq < 875)) freq = 1045;
  
  pinMode (DAT, OUTPUT);
  pinMode (SH,  OUTPUT);
  pinMode (ST,  OUTPUT);
  
  pinMode (L1, OUTPUT);
  pinMode (L2, OUTPUT);
  pinMode (L3, OUTPUT);
  pinMode (L4, OUTPUT);
  pinMode (L5, OUTPUT);
  pinMode (L6, OUTPUT);

  pinMode (A, OUTPUT);
  pinMode (B, OUTPUT);
  pinMode (C, OUTPUT);
  pinMode (D, OUTPUT);

  cli(); // отключить глобальные прерывания

  TCCR1A = 0;
  TCCR1B = 0;
  TIMSK1 = 0;

  OCR1A = 1250; // установка регистра совпадения

  TCCR1A = (0 << COM1A1) | (0 << COM1A0) | (0 << COM1B1) | (0 << COM1B0) | (1 << WGM11) | (1 << WGM10);
  TCCR1B = (1 << WGM12) | (1 << WGM13) | (0 << CS12) | (0 << CS11) | (1 << CS10); // включение в CTC режим

  TIMSK1 = (0 << ICIE1) | (0 << OCIE1B) | (1 << OCIE1A) | (0 << TOIE1);

  sei ();

  setup_radio_first ();
  set_volume (main_volume);
  delay (500);
  setFreq (freq);
  delay (500);
  switch_on_radio ();
  freq_to_lamps ();
}

unsigned char lb = 0;
unsigned char rb = 0;

void set_bytes ()  {
  switch (main_volume) {
    case 0: lb = 0b00100000;
            rb = 0b00000000;
            break;
    case 1: lb = 0b00110000;
            rb = 0b00000000;   
            break;
    case 2: lb = 0b00111000;
            rb = 0b00000000; 
            break;    
    case 3: lb = 0b00111100;
            rb = 0b00000000; 
            break; 
    case 4: lb = 0b00111110;
            rb = 0b00000000; 
            break;   
    case 5: lb = 0b00111111;
            rb = 0b00000000; 
            break;  
    case 6: lb = 0b00111111;
            rb = 0b10000000; 
            break;  
    case 7: lb = 0b00111111;
            rb = 0b11000000; 
            break; 
    case 8: lb = 0b00111111;
            rb = 0b11100000; 
            break;      
    case 9: lb = 0b00111111;
            rb = 0b11110000; 
            break;  
    case 10:lb = 0b00111111;
            rb = 0b11111000; 
            break;        
    case 11:lb = 0b00111111;
            rb = 0b11111100; 
            break; 
    case 12:lb = 0b00111111;
            rb = 0b11111110; 
            break;  
    case 13:lb = 0b00111111;
            rb = 0b11111111; 
            break;              
  }
}

bool first_run = 1;
void loop() {

  if (first_run) {
    set_volume (main_volume);
    delay (500);
    setFreq (freq);    
    delay (500);
    switch_on_radio ();
    delay (500);
    first_run = 0;
  }

  set_bytes ();
 
  digitalWrite (ST, LOW);
  shiftOut (DAT, SH, MSBFIRST, lb);
  shiftOut (DAT, SH, MSBFIRST, rb);
  digitalWrite (ST, HIGH);  
  
  enc2.tick();
  enc1.tick();

  if (enc1.right()) {
    if (main_volume > 0) main_volume--;
  }
  if (enc1.left()) {
    if (main_volume < 13) main_volume++;
  }
  if (enc1.click ()) {
    set_volume (main_volume);
    eeprom_write_byte (&main_volume_addr, main_volume);
  }
    
  if (enc2.left()) {
    if (freq < 1080) freq++;
    freq_to_lamps ();
  }
  if (enc2.right()) {
    if (freq > 875) freq--;
    freq_to_lamps ();
  }

  if (enc2.click ()) {
    setFreq (freq);
    switch_on_radio ();
    eeprom_write_word (&freq_addr, freq);
  }
}

void freq_to_lamps () {
    dig [0] = (freq / 1000) % 10;
    dig [1] = (freq / 100) % 10;
    dig [2] = (freq / 10) % 10;
    dig [3] = freq % 10;
}

void print_lamp (unsigned char num) {
  switch (num) {
    case 0:
      digitalWrite (A, 1); // ok
      digitalWrite (B, 1);
      digitalWrite (C, 0);
      digitalWrite (D, 0);
      break;
    case 1:
      digitalWrite (A, 0); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 1);
      digitalWrite (D, 0);
      break;
    case 2:
      digitalWrite (A, 1); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 1);
      digitalWrite (D, 0);
      break;
    case 3:
      digitalWrite (A, 1);
      digitalWrite (B, 0);
      digitalWrite (C, 1);
      digitalWrite (D, 1);
      break;
    case 4:
      digitalWrite (A, 0); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 0);
      digitalWrite (D, 1);
      break;
    case 5:
      digitalWrite (A, 1); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 0);
      digitalWrite (D, 0);
      break;
    case 6:
      digitalWrite (A, 0); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 0);
      digitalWrite (D, 0);
      break;
    case 7:
      digitalWrite (A, 1); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 0);
      digitalWrite (D, 1);
      break;
    case 8:
      digitalWrite (A, 0); // ok
      digitalWrite (B, 0);
      digitalWrite (C, 1);
      digitalWrite (D, 1);
      break;
    case 9:
      digitalWrite (A, 0); // ok
      digitalWrite (B, 1);
      digitalWrite (C, 0);
      digitalWrite (D, 0);
      break;
    case 10:
      digitalWrite (A, 1);
      digitalWrite (B, 1);
      digitalWrite (C, 1);
      digitalWrite (D, 1);
      break;
  }
}

ISR(TIMER1_COMPA_vect)
{
  switch (a) {
    case 0:
      print_lamp (dig [0]);
      digitalWrite (L1, 1);
      break;
    case 1:
      print_lamp (dig [1]);
      digitalWrite (L2, 1);
      break;
    case 2:
      print_lamp (dig [2]);
      digitalWrite (L3, 1);
      break;
    case 3:
      print_lamp (dig [3]);
      digitalWrite (L4, 1);
      break;
    case 4:
      print_lamp (dig [4]);
      digitalWrite (L5, 1);
      break;
    case 5:
      print_lamp (dig [5]);
      digitalWrite (L6, 1);
      break;
  }
  e++;
  if (e > 5) {
    e = 0;

    digitalWrite (L1, 0);
    digitalWrite (L2, 0);
    digitalWrite (L3, 0);
    digitalWrite (L4, 0);
    digitalWrite (L5, 0);
    digitalWrite (L6, 0);
    print_lamp (10);

    a++;
    if (a > 5) {
      a = 0;
    }
  }
  TCNT1H = 0x00;
  TCNT1L = 0x00;
}

ISR(TIMER1_OVF_vect)
{

}
