#define RDA5807M_RANDOM_ACCESS_ADDRESS 0x11
// регистры
#define RDA5807M_REG_CONFIG 0x02
#define RDA5807M_REG_TUNING 0x03
#define RDA5807M_REG_VOLUME 0x05
#define RDA5807M_REG_RSSI   0x0B
// флаги
#define RDA5807M_FLG_DHIZ 0x8000
#define RDA5807M_FLG_DMUTE 0x4000
#define RDA5807M_FLG_BASS 0x1000
#define RDA5807M_FLG_ENABLE word(0x0001)
#define RDA5807M_FLG_TUNE word(0x0010)
//маски
#define RDA5807M_CHAN_MASK 0xFFC0
#define RDA5807M_CHAN_SHIFT 6
#define RDA5807M_VOLUME_MASK word(0x000F)
#define RDA5807M_VOLUME_SHIFT 0
#define RDA5807M_RSSI_MASK 0xFE00
#define RDA5807M_RSSI_SHIFT 9

uint8_t volume = 0; // 0..15
uint16_t reg02h, reg03h, reg05h, reg0Bh;

unsigned char vol = 0;

void setRegister(uint8_t reg, const uint16_t value) {
  Wire.beginTransmission(0x11);
  Wire.write(reg);
  Wire.write(highByte(value));
  Wire.write(lowByte(value));
  Wire.endTransmission(true);
}

uint16_t getRegister(uint8_t reg) {
  uint16_t result;
  Wire.beginTransmission(RDA5807M_RANDOM_ACCESS_ADDRESS);
  Wire.write(reg);
  Wire.endTransmission(false);
  Wire.requestFrom(0x11, 2, true);
  result = (uint16_t)Wire.read() << 8;
  result |= Wire.read();
  return result;
}

void setup_radio_first (void) {
  Wire.begin();
  // Регистр 02h - включение, настройки
  reg02h = RDA5807M_FLG_ENABLE | RDA5807M_FLG_DHIZ | RDA5807M_FLG_DMUTE;
  setRegister(RDA5807M_REG_CONFIG, reg02h);

  // А потом решили еще усилить басы:
  reg02h |= RDA5807M_FLG_BASS;
  setRegister(RDA5807M_REG_CONFIG, reg02h);

  // Регистр 03h - выбор радиостанции
  // После сброса в регистре 03h значение по умолчанию - 0
  // Таким образом BAND = 00 (87..108МГц), STEP = 00 (100кГц). Оставим их как есть
  reg03h = (freq - 870) << RDA5807M_CHAN_SHIFT; // chan = (freq - band) / space
  setRegister(RDA5807M_REG_TUNING, reg03h | RDA5807M_FLG_TUNE);

  // Регистр 05h. Установим громкость, остальные биты не изменяем
  reg05h = getRegister(RDA5807M_REG_VOLUME); // Считываем текущее значение
  reg05h &= ~RDA5807M_VOLUME_MASK; // Сбрасываем биты VOLUME
  reg05h |= volume << RDA5807M_VOLUME_SHIFT; // Устанавливаем новую громкость
  setRegister(RDA5807M_REG_VOLUME, reg05h);
  Wire.end ();
}

void shutdown_radio (void) {
  reg02h = 0 | RDA5807M_FLG_DHIZ | 0;
  setRegister(RDA5807M_REG_CONFIG, reg02h);
}

void switch_on_radio () {
  reg02h = RDA5807M_FLG_ENABLE | RDA5807M_FLG_DHIZ | RDA5807M_FLG_DMUTE;
  setRegister(RDA5807M_REG_CONFIG, reg02h);

  reg03h = (freq - 870) << RDA5807M_CHAN_SHIFT; // chan = (freq - band) / space
  setRegister(RDA5807M_REG_TUNING, reg03h | RDA5807M_FLG_TUNE);
}

void set_volume (unsigned char volume) {
  // Регистр 05h. Установим громкость, остальные биты не изменяем
  reg05h = getRegister(RDA5807M_REG_VOLUME); // Считываем текущее значение
  reg05h &= ~RDA5807M_VOLUME_MASK; // Сбрасываем биты VOLUME
  reg05h |= volume << RDA5807M_VOLUME_SHIFT; // Устанавливаем новую громкость
  setRegister(RDA5807M_REG_VOLUME, reg05h);
}

void setFreq (int f) {
  int freq = f;
  Wire.begin ();
  // Регистр 03h - выбор радиостанции
  // После сброса в регистре 03h значение по умолчанию - 0
  // Таким образом BAND = 00 (87..108МГц), STEP = 00 (100кГц). Оставим их как есть
  reg03h = (freq - 870) << RDA5807M_CHAN_SHIFT; // chan = (freq - band) / space
  setRegister(RDA5807M_REG_TUNING, reg03h | RDA5807M_FLG_TUNE);
  Wire.end ();
}
/*
uint16_t control_freq (uint16_t f) {
  uint16_t tmp;
  if (f > 1080) tmp = 1080;
  if (f < 875) tmp = 875;
  return tmp; 
}*/
